// AtoD.cpp: implementation of the CAtoD class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "AtoD.h"
#include "crc.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAtoD::CAtoD()
{
	int i;            //loop control
    channels = 0x01;  //Default channel is a

	preset = 0x00;     //no presets

	for (i=0; i<9; i++)
	{
	 page0[i] = 0x00;	  //empty page readouts
	 page1[i] = 0x00;
	 page2[i] = 0x00;
	}

    for (i=0; i<4; i++)   //clear voltage readings
	{
		voltage[i] = 0;
	}

	makeConnection();
	
	readPage(0);
	readPage(1);
	readPage(2);

	//clear POR bit
	page1[1] = page1[1] & 0x7F;
	page1[3] = page1[3] & 0x7F;
	page1[5] = page1[5] & 0x7F;
	page1[7] = page1[7] & 0x7F;

	writePage(1);

}

CAtoD::~CAtoD()
{

}

/////////////////////////////////////////////////////////////////////////////
// Private Method Definitons

int CAtoD::makeConnection()
{
   short printmsg=1;      //flag used as boolean to test whether message should be printed       
   short done =0;         //flag to indicate TMEX session complete
   int retval;            //return value
   //attempt to get port number and port type from registry
   if (readPortFromRegistry() != 1)
   {
	  retval = ATD_PORT_NUM_NOT_FOUND;
	  done = 1;
   }

   //attempt to get version and driver information
   if (displayHeader() < 0)
   {
	  retval = ATD_NO_HARDWARE_DRIVER;
	  done =1;
   }

   //loop to get a session handle and set up MicroLan
   while ( !done ) 
   {
      // attempt to get a session 
      s_handle = TMExtendedStartSession(PortNum,PortType,NULL);
	  
	  //if session fails
	  if (s_handle < 0)
	  {
		  retval = ATD_NO_HARDWARE_DRIVER;
		  done =1;
	  }
	  //else if session ok
	  else if (s_handle > 0)
	  {
		  //attempt setup 
		  if (TMSetup(s_handle) != 1) //setup fails
		  {
			  retval = ATD_MICROLAN_SETUP_FAIL;
			  done = 1;
		  }
		  //attempt to find a DS2450
		  else if (!FindFirstFamily(0x20)) //find fails
		  {
			  retval = ATD_NO_DS2450_ON_MICROLAN;
			  done = 1;
		  }
		  else //setup ok, and found button
		  {
			  retval = 1;
			 done = 1;
		  }
	  }
	  //if waiting for a session, do nothing (s_handle == 0)
	}
 
  // end the session if one is open 
  TMEndSession(s_handle);
  return retval;
}

//---------------------------------------------------------------------------------
//readPortFromRegistry
//in: void
//gets default port type and port number from the registry
//out: 1= success, <0 unable to read type
int CAtoD :: readPortFromRegistry()
{
   char ReturnString[256]; //Hold the returned port number/type
   HKEY GlobKey;		   //key for getting system registry information
   DWORD dwType,size;      //system information variables

   // read the default PortNum and PortType from the registry 
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
         "Software\\Dallas Semiconductor\\iButton TMEX\\3.00",0,
         KEY_READ,&GlobKey) == ERROR_SUCCESS)
   {
      // attempt to read the PortNum 
      size = 255;
      PortNum = 1;

      if (RegQueryValueEx(GlobKey,"DefaultPortNum",NULL,&dwType,
          (unsigned char *) &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortNum = atoi(ReturnString);
      }
	  else
	  {
		  RegCloseKey(GlobKey);
		  return ATD_PORT_NUM_NOT_FOUND;
	  }
      // attempt to read the PortType 
      size = 255;
      PortType = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortType",NULL,&dwType,
          (unsigned char *)&ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortType = atoi(ReturnString);
      }
	  else 
	  {
		  RegCloseKey(GlobKey);
		  return ATD_PORT_TYPE_NOT_FOUND;
	  }

      // close the key
      RegCloseKey(GlobKey);
	  return 1;
   }
   else
	   return -1;
     
}

//------------------------------------------------------------------------
//displayHeader
//in: void
//attempts to get version of TMEX and of drivers
//out: 1 if drivers found, <1 if drivers not found
int CAtoD :: displayHeader()
{
	 char tstr[80];       //holds type version information
	 char vstr[80]; 	  //holds TMEX version information
	 int retval;          //return value from TmGetTypeVersion

   Get_Version(vstr);
   retval = TMGetTypeVersion(PortType,tstr);
   sprintf(driverInfo,"Dallas Semiconductor Corporation\nCopyright (C) 1992-1999\n"
		          "Port number: %d Port type: %d\n\nMain Driver: %s\nType%d: %s\n\n\n", 
	       		  PortNum, PortType,vstr,PortType,tstr); 
   
   if (retval < 0 )
	   return ATD_NO_TYPE_VERSION_INFO_FOUND;
   else 
	   return 1;
}



//----------------------------------------------------------------------
// Find the first device with a particular family
 
short CAtoD :: FindFirstFamily(short family)
{
   short j;
   char byteStr[4] = {0};
  
   // set up to find the first device with family 'family' 
   if (TMFamilySearchSetup(s_handle,state_buffer,family) == 1)
   { 
      // get first device in list with the specified family 
      if (TMNext(s_handle,state_buffer) == 1)
      {
         // read the rom number 
         ROM[0] = 0;
         TMRom(s_handle,state_buffer,ROM);   
         
         // check if correct type 
		 sprintf(ROMStr,"Serial ROM ID: ");
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
             for (j = 7; j >= 0; j--)
			 {
				sprintf(byteStr,"%02X",ROM[j]);
				strcat(ROMStr,byteStr);
			 }
			 return 1;
         }        
      }
   }
    
   // failed to find device of that type 
   return ATD_NO_DS2450_ON_MICROLAN;                              
}
//------------------------------------------------------------------------
//readPage
//reads a page from the iButton
//returns 1 if successful, <0 otherwise
int CAtoD::readPage(short pageno)
{
	uchar * ppage;
	short flag;

	s_handle = TMExtendedStartSession(PortNum,PortType,NULL);
	if ( s_handle < 1)
	{
		return ATD_NO_MICROLAN_SESSION;
	}

    switch (pageno)
	{
	case 0:
		ppage = &page0[0];
		break;
	case 1:
		ppage = &page1[0];
		break;
	case 2:
		ppage = &page2[0];
		break;
	default:
		return ATD_UNAVAILABLE_PAGE;
	}

		
	flag = TMExtendedReadPage(s_handle,state_buffer,pageno,ppage,0);
	if (flag <= 0)
	{
		TMEndSession(s_handle);
		return ATD_ERROR_READING_PAGE;
	}

	TMEndSession(s_handle);
	return 1;
}//end readpage



//-----------------------------------------------------------------------
// writePage
// sends the current specified page to the iButton 	
// out: returns 1 if successful, <0 if not
int CAtoD::writePage(short pageno)
{
	//to write 8 bytes to the given page, and
	//to use the TMBlockIO() function to write this data, we need 45 bytes in 
	//the message buffer.
	
	short pktlen = 0;        //length of packet to be sent
    uchar* pageToWrite;      //pointer to correct page
	uchar packet[46] ={0};   //buffer to hold packet data
    int i, j;                //loop control
	short numsent =0;        //number of bytes read/written by TMBlockIO 
	ushort currentaddr;      //current write address for the 2450

	//get correct page
	switch(pageno)
	{
	case 1:
		pageToWrite = &page1[0];
		currentaddr = 0x0008;   //address of first control memory byte
		break;
	case 2:
		pageToWrite  = &page2[0];
		currentaddr = 0x0010;   //address of first alarm settings memory byte
		break;
	default:
		return ATD_UNAVAILABLE_PAGE;
	}
	
	//create packet
	packet[pktlen++] = 0x55; //MatchROM command

	for (i=0; i<8; i++)
		packet[pktlen++] = (uchar) ROM[i];  //ROM value for MatchROM

	packet[pktlen++] = 0x55;                       //WriteMemory Command
	packet[pktlen++] = (uchar) currentaddr;        //address of first byte
	packet[pktlen++] = 0x00;                       //address of first byte

	for ( j=0; j < 8 ; j++) //for each byte to be sent
	{
		packet[pktlen++] = pageToWrite[j];    //set byte to send
		packet[pktlen++] = 0xFF;			 //read CRC
		packet[pktlen++] = 0xFF;		     //read CRC
		packet[pktlen++] = 0xFF;             //read data confirmation byte
	}

	//attempt to get session
	s_handle = TMExtendedStartSession(PortNum,PortType,NULL);
	if (s_handle < 0)
	{
		return ATD_NO_HARDWARE_DRIVER;
	}


	//send data
	numsent = TMBlockIO(s_handle, &packet[0], pktlen);
    if (numsent != pktlen)
	{
		TMEndSession(s_handle);
		return ATD_UNABLE_TO_WRITE_PAGE;
	}

	//end session
	TMEndSession(s_handle);

	//Check CRC's

	CRC16 = 0;              //clear CRC before first write
	docrc16(0x55);         //write command
	docrc16((uchar) currentaddr);         //address
	docrc16(0x00);         //address
	docrc16(pageToWrite[0]);   //first data byte
	docrc16(packet[13]);   //first CRC byte
	docrc16(packet[14]);   //second CRC byte

	if (CRC16 != CRC16_OK)
	{
		return ATD_FAILED_CRC_CHECK;
	}

	for (j=1; j<8; j++)  //for each subsequent byte written
	{
		currentaddr++; 
		
		CRC16 = currentaddr;      //seed CRC with address written
		docrc16(pageToWrite[j]);  //shift in next data byte
		i = 13+(4*j);             //index into packet where CRC is returned
		if (i+1 > pktlen)
		{
			return ATD_FAILED_CRC_CHECK;
		}

		docrc16(packet[i]);       //shift in first CRC byte
		docrc16(packet[i+1]);     //shift in second CRC byte
		
		if (CRC16 != CRC16_OK)
		{
	    	return ATD_FAILED_CRC_CHECK;
		}
	}
	return 1;
}//end writePage

//----------------------------------------------------------------------------------
//readingToVoltage
//converts the 16 bit conversion data into voltage
//returns the voltage

float CAtoD::readingToVoltage(uchar mychannel)
{
	//contains  2^x in the x th index
	int PowersOfTwo[17] = {1,2,4,8,16,32,64,128,256,512,1024,2048,4096,8192,16384,32768,65536};
	uchar IRMask = 0x01;
	uchar RCMask = 0x0F;

	if (forceSingleChannel(mychannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	float range;
	int resolution; 
    int addr;
	float increment;
	unsigned int reading;
	float voltage;
	
	//assign appropriate starting address for the channel selected
	addr = getAddr(mychannel);
	if (addr <0)
		return -1;
	
	//get resolution from control page
	if ( (RCMask & page1[addr]) == 0x00)
		resolution = 16;
	else 
		resolution = (RCMask & page1[addr]);
	

	//get range from control page
	if ( (IRMask & page1[addr+1]) == 0)
		range = 2.56;
	else 
		range = 5.12;

	increment = range / PowersOfTwo[resolution];  //find increment of the reading
	reading = page0[addr+1];			    //reading now contains read-out high byte in its low order byte
	reading = reading << 8;	    //left shift 8 bits; read-out high byte is now in high order byte
    reading += page0[addr];          //add in low order byte
	reading = reading >> (16-resolution); //right shift to get 
     									  //correct resolution on reading
	voltage = reading * increment;  //calculate real voltage

    return voltage;
}


//---------------------------------------------------------------------------------
//forceSingleChannel
//returns 1 if a single channel is selected, <0 otherwise
int CAtoD::forceSingleChannel(uchar forChannel)
{
	if ((forChannel == 0x01) || (forChannel == 0x02) || (forChannel == 0x04) || (forChannel == 0x08))
		return 1;
	else 
		return -1;
}

//-----------------------------------------------------------------------------------
//getAddr
//returns the offset in to a memory page for the given channel
uchar CAtoD::getAddr(uchar forChannel)
{
	uchar addr;

	switch(forChannel)
	{
	case 1:
		addr =0;
		break;
	case 2:
		addr =2;
		break;
	case 4:
		addr =4;
		break;
	case 8:
		addr =6;
		break;
	default:
		return 0xFF;
	}
	return addr;
}

/////////////////////////////////////////////////////////////////////////////
//Public Method Definitions

//---------------------------------------------------------------------------
//DumpPage
//returns a string in the given char array
//  the given char array should be at least 40 charactrs long
//returns 1 if successful, <0 if not
int CAtoD::DumpPage(short pageno, char *pageStr)
{
	uchar * ppage;
	int i;
	char byteStr[4];   //string to write individual byte
	switch (pageno)
	{
	case 0:
		ppage = &page0[0];
		break;
	case 1:
		ppage = &page1[0];
		break;
	case 2:
		ppage = &page2[0];
		break;
	default:
		return -1;
	}

	readPage(pageno);
	sprintf(pageStr,"Page %d:\n",pageno);
	for (i=0; i <8; i++)
	{
		sprintf(byteStr,"%02x ", ppage[i]);
		strcat(pageStr,byteStr);
	}
	return 1;
}

//--------------------------------------------------------------------
//ReadVoltage
//reads the current voltages as specified 
//returns 1 if successful, <0 otherwise
int CAtoD::ReadVoltage()
{
	//write control to 2450, read that into page 1
	writePage(1);
	readPage(1);
	
	short pktlen = 0;   //length of packet to be sent
	uchar packet[13];   //buffer to hold packet data
    int i;              //loop control
	short numsent =0;   //number of bytes read/written by TMBlockIO 
	uchar lastCRC;      //buffer to hold last byte of CRC info
	unsigned long st;   //timer to hold strong pullup
	

	packet[pktlen++] = 0x55; //MatchROM command

	for (i=0; i<8; i++)
		packet[pktlen++] = (uchar) ROM[i];  //ROM value for MatchROM

	packet[pktlen++] = 0x3C;		               //convert Command
	packet[pktlen++] = channels;                   //input select mask
	packet[pktlen++] = preset;                     //read out control
	packet[pktlen++] = 0xFF;			           //read CRC, first byte

	//getSession
	s_handle = TMExtendedStartSession(PortNum,PortType,NULL);
	if (s_handle <0)
	{
		return ATD_NO_MICROLAN_SESSION;
	}

	//split up reading CRC to ensure that pullup occurs in the 10 microsecond window
    numsent = TMBlockIO(s_handle, &packet[0], pktlen); //send packet
	if (numsent != pktlen)							   //check send
	{
		TMEndSession(s_handle);
		return ATD_UNABLE_TO_WRITE_PAGE;
	}

    //send strong pullup after next byte
	TMOneWireLevel(s_handle,LEVEL_SET,LEVEL_STRONG_PULLUP,PRIMED_BYTE); 
															
	lastCRC = TMTouchByte(s_handle,0xFF);//readCRC, second byte

	//checkCRC
	CRC16 = 0;				//clear CRC 
	for (i=9; i<13; i++)    //convert command is in 9th byte of the packet so start there
		docrc16(packet[i]); //13 is end of packet
	docrc16(lastCRC);
    if (CRC16 != CRC16_OK)
	{
		TMOneWireLevel(s_handle,LEVEL_SET,LEVEL_NORMAL,PRIMED_NONE); //release pullup before return
		TMEndSession(s_handle);
		return ATD_FAILED_CRC_CHECK;
	}

	//wait for conversion; max wait will be 5.28 microseconds for 16 bit conversions
	//on all four channels
	st = GetTickCount() + 100; //.1 sec timer
	while (GetTickCount() < st)
		TMValidSession(s_handle);

	//disable strong pullup
	TMOneWireLevel(s_handle,LEVEL_SET,LEVEL_NORMAL,PRIMED_NONE);
	TMAccess(s_handle,&state_buffer);
	TMEndSession(s_handle);

	//get voltage and alarm readings from the 2450
	readPage(0);
	readPage(1);

	//convert readings to voltages
	voltage[0] = readingToVoltage(0x01);
	voltage[1] = readingToVoltage(0x02);
	voltage[2] = readingToVoltage(0x04);
	voltage[3] = readingToVoltage(0x08);

	return 1;
}


//-----------------------------------------------------------------
//getChannels
//returns booleans indicating which channels are active

void CAtoD::getChannels(BOOL& A, BOOL& B, BOOL& C, BOOL& D)
{
	A = B = C = D = FALSE;
	
	uchar mask = 0x01;
	if ((mask & channels) > 0 )
		A = TRUE;

	mask = 0x02;
	if ((mask & channels) > 0 )
		B = TRUE;

	mask = 0x04;
	if ((mask & channels) > 0 )
		C = TRUE;

	mask = 0x08;
	if ((mask & channels) > 0 )
		D = TRUE;
	
}

//-------------------------------------------------------------------
//setChannels
//sets the active channels
int CAtoD::setChannels(BOOL A, BOOL B, BOOL C, BOOL D)
{
	if (A)
		channels = channels | 0x01;
	else
		channels = channels & 0x0E;

	if (B)
		channels = channels | 0x02;
	else
		channels = channels & 0x0D;

	if (C)
		channels = channels | 0x04;
	else
		channels = channels & 0x0B;

	if (D)
		channels = channels | 0x08;
	else
		channels = channels & 0x07;

	return 1;
}


//---------------------------------------------------------------------------
//getVoltageReading
//
float CAtoD::getVoltageReading(uchar forChannel)
{
	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	int index;
	switch (forChannel)
	{
	case 0x01:
		index = 0;
		break;
	case 0x02:
		index = 1;
		break;
	case 0x04:
		index = 2;
		break;
	case 0x08:
		index = 3;
		break;
	}

	return voltage[index];
}

//---------------------------------------------------------------------------------
//getResolution
//returns the number of bits of resolution for the given channel
uchar CAtoD::getResolution(uchar forChannel)
{
	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	uchar RCMask = 0x0F;
	int addr;
	uchar resolution;

	//assign appropriate starting address for the channel selected
	addr = getAddr(forChannel);
	if (addr > 8)
		return -1;

	//get resolution from control page
	if ( (RCMask & page1[addr]) == 0x00)
		resolution = 16;
	else 
		resolution = (RCMask & page1[addr]);
	return resolution;
}

//----------------------------------------------------------------------
//setResolution
//sets the resolution for the given channel
//returns 1 if succesful, <0 otherwise
int CAtoD::setResolution(uchar forChannel, uchar newResolution)
{
	uchar addr;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	if ((newResolution == 0) || (newResolution >16))
	{
		return ATD_RESOLUTION_OUT_OF_BOUNDS;
	}

	//sixteen bit resolution requires 0000 in the RC bits
	if (newResolution == 16)
		newResolution = 0;

	//assign appropriate starting address for the channel selected
	addr = getAddr(forChannel);
	if (addr > 8)
		return -1;
	
	page1[addr] = page1[addr] >> 4;
	page1[addr] = page1[addr] << 4;
	page1[addr] = page1[addr] | newResolution;

	writePage(1);
	return 1;
}

//--------------------------------------------------------------------
//getOutputEnable 
//TRUE is an input channel; FALSE is an output channel
//returns 1 if successful, <0 otherwise
int CAtoD::getOutEnable(uchar forChannel, BOOL& on, BOOL& transistorOff)
{
	int addr;
	uchar outMask = 0x80;
	uchar transMask = 0x40;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr >8)
		return -1;

	on = ((page1[addr] & outMask) == outMask);
	transistorOff = ((page1[addr] & transMask) == transMask);

	return 1;
}


//-------------------------------------------------------------------------
//setOutputEnable
//sets the output enable and output control for the 2450
//returns 1 if sucessful, <0 otherwise
int CAtoD::setOutputEnable(uchar forChannel,BOOL on,BOOL transistorOff)
{
	int addr;
	uchar OEMask = 0x00;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr > 8)
		return -1;
	
	if (on)
	{
		if (transistorOff)
			OEMask = 0xC0;
		else 
			OEMask = 0x80;
	}

	page1[addr] = ((page1[addr] & 0x0F) | OEMask);
	writePage(1);
	return 1;
}

//--------------------------------------------------------------------------------
//getAlarmH
//returns the value of the high alarm enable flag (AHE) (True if set, false otherwise)
BOOL CAtoD::getAlarmH(uchar forChannel)
{
	int addr;
	uchar AEHMask = 0x08;

	if (forceSingleChannel(forChannel) <0)
	{
		return FALSE;
	}

	addr = getAddr(forChannel);
	if (addr > 8)
		return FALSE;
	
	if ((page1[addr+1] & AEHMask) >0)
		return TRUE;
	else 
		return FALSE;
}


//----------------------------------------------------------------
//int setAlarmH
//sets the alarm high enable flag for the given channel
//returns 1 if successful, 0 otherwise
int CAtoD::setAlarmH(uchar forChannel,BOOL on)
{
	int addr;
	uchar AEHMask;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr > 8)
		return -1;

	if (on)
		AEHMask = 0x08;
	else 
		AEHMask = 0x00;

	page1[addr+1] = ((page1[addr+1] & 0xF7) | AEHMask);
	writePage(1);
	return 1;
}

//----------------------------------------------------------------------------
//getAlarmHFlag
//returns the value of the AFH bit; if it cannot, it returns false
BOOL CAtoD::getAlarmHFlag(uchar forChannel)
{
	int addr;
	uchar AFHMask = 0x20;

	if (forceSingleChannel(forChannel) <0)
	{
		return FALSE;
	}

	addr = getAddr(forChannel);
	if (addr >8 )
		return FALSE;
	
	if ((page1[addr+1] & AFHMask) >0)
		return TRUE;
	else 
		return FALSE;
}


//----------------------------------------------------------------
//int setAlarmHFlag
//sets the alarm high flag for the given channel
//returns 1 if successful, 0 otherwise
int CAtoD::setAlarmHFlag(uchar forChannel,BOOL flag)
{
	int addr;
	uchar AFHMask;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr > 8 )
		return -1;
	
	if (flag)
		AFHMask = 0x20;
	else 
		AFHMask = 0x00;

	page1[addr+1] = ((page1[addr+1] & 0xDF) | AFHMask);
	writePage(1);
	return 1;
}

//----------------------------------------------------------------------------------
//getAlarmHSetting
//returns the alarm setting for the specified channel
uchar CAtoD::getAlarmHSetting(uchar forChannel)
{
	int addr;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr >8)
		return 0;

	readPage(2);
	return page2[addr+1];
}

//----------------------------------------------------------------------------------
//setAlarmHSetting
//sets the Alarm high threshold for the given channel
//returns 1 for success, <0 otherwise
int CAtoD::setAlarmHSetting(uchar forChannel, uchar setting)
{
	int addr;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr >8 )
		return -1;

	readPage(2);
	page2[addr+1] = setting;
	writePage(2);
	return 1;
}


//----------------------------------------------------------------------------------
//getAlarmL
//returns the value of the AEL bit
BOOL CAtoD::getAlarmL(uchar forChannel)
{
	int addr;
	uchar AELMask = 0x04;

	if (forceSingleChannel(forChannel) <0)
	{
		return FALSE;
	}

	addr = getAddr(forChannel);
	if (addr >8)
		return addr;

	if ((page1[addr+1] & AELMask) >0)
		return TRUE;
	else 
		return FALSE;
}


//----------------------------------------------------------------
//sets the alarm low enable flag for the given channel
//returns 1 if successful, 0 otherwise
int CAtoD::setAlarmL(uchar forChannel,BOOL on)
{
	int addr;
	uchar AELMask;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr >8 )
		return -1;

	if (on)
		AELMask = 0x04;
	else 
		AELMask = 0x00;

	page1[addr+1] = ((page1[addr+1] & 0xFB) | AELMask);
	writePage(1);
	return 1;
}

//----------------------------------------------------------------------------
//getAlarmLFlag
//returns the value of the AFL bit
BOOL CAtoD::getAlarmLFlag(uchar forChannel)
{
	int addr;
	uchar AFLMask = 0x10;

	if (forceSingleChannel(forChannel) <0)
	{
		return FALSE;
	}

	addr = getAddr(forChannel);
	if (addr >8)
		return FALSE;
	
	if ((page1[addr+1] & AFLMask) >0)
		return TRUE;
	else 
		return FALSE;
}


//----------------------------------------------------------------
//int setAlarmLFlag
//sets the alarm low flag for the given channel
//returns 1 if successful, 0 otherwise
int CAtoD::setAlarmLFlag(uchar forChannel,BOOL flag)
{
	int addr;
	uchar AFLMask;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr > 8)
		return -1;

	if (flag)
		AFLMask = 0x10;
	else 
		AFLMask = 0x00;

	page1[addr+1] = ((page1[addr+1] & 0xEF) | AFLMask);
	writePage(1);
	return 1;
}

//----------------------------------------------------------------------------------
//getAlarmLSetting
//returns the alarm setting for the specified channel
uchar CAtoD::getAlarmLSetting(uchar forChannel)
{
	int addr;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr >8)
		return 0;

	readPage(2);
	return page2[addr];
}

//----------------------------------------------------------------------------------
//setAlarmLSetting
//sets the Alarm low threshold for the given channel
//returns 1 for success, <0 otherwise
int CAtoD::setAlarmLSetting(uchar forChannel, uchar setting)
{
	int addr;

	if (forceSingleChannel(forChannel) <0)
	{
			return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr >8 )
		return -1;

	readPage(2);
	page2[addr] = setting;
	writePage(2);
	return 1;
}

//--------------------------------------------------------------------------
//getVoltageRange
//returns the IR flag
//true if voltage is set to 5.12 max, false if voltage is set to 2.56 max
BOOL CAtoD::getVoltageRange(uchar forChannel)
{
	int addr;
	uchar VMask = 0x01;

	if (forceSingleChannel(forChannel) <0)
	{
		return FALSE;
	}

	addr = getAddr(forChannel);
	if ( addr >8)
		return FALSE;

	if ((page1[addr+1] & VMask) >0)
		return TRUE;
	else 
		return FALSE;
}

//-----------------------------------------------------------------------
//setVoltageRange
//sets the voltage for the given channel; TRUE indicates a max of 5.12 V,
//FALSE indicates an max of 2.56 V
//returns 1 if successful, <0 otherwise
int CAtoD::setVoltageRange(uchar forChannel, BOOL range)
{
	int addr;
	uchar VMask;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	addr = getAddr(forChannel);
	if (addr > 8)
		return -1;

	if (range)
		VMask = 0x01;
	else 
		VMask = 0x00;

	page1[addr+1] = ((page1[addr+1] & 0xFE) | VMask);

	writePage(1);
	return 1;
}

//---------------------------------------------------------------------------
//getPreset
//returns the preset for the given channel
//NOTE: this preset is the one given in the 2450 spec:
//for a given channel, 00=no preset; 01=preset to 0; 10=preset to 1; 11=illegal
//returns 1 if successful, <0 otherwise
int CAtoD::getPreset(uchar forChannel, BOOL& set, BOOL& clear)
{
	uchar ChMask;
	uchar rightShift;
	uchar ChPreset;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	switch(forChannel)
	{
	case 1:
		ChMask =0x03;
		rightShift =0;
		break;
	case 2:
		ChMask =0x0C;
		rightShift =2;
		break;
	case 4:
		ChMask =0x30;
		rightShift =4;
		break;
	case 8:
		ChMask =0xC0;
		rightShift =6;
		break;
	default:
		return ATD_INCORRECT_CHANNEL;
	}

	ChPreset = (preset & ChMask) >> rightShift;
	if (ChPreset == 0)
	{
		set = FALSE;
		clear = FALSE;
	}
    else if (ChPreset == 1)
	{
		set = FALSE;
		clear = TRUE;
	}
	else if (ChPreset == 2)
	{
		set = TRUE;
		clear = FALSE;
	}
	else //illegal value
	{
		return ATD_ILLEGAL_PRESET;
	}
	return 1;
}

//---------------------------------------------------------------------------
//setPreset
//sets the preset bits for the given channel
//NOTE: this preset is the one given in the 2450 spec:
//for a given channel, 00=no preset; 01=preset to 0; 10=preset to 1; 11=illegal
//returns 1 if successful, <0 otherwise
int CAtoD::setPreset(uchar forChannel, BOOL set, BOOL clear)	
{
	uchar ChMask;

	if (forceSingleChannel(forChannel) <0)
	{
		return ATD_MULTIPLE_CHANNELS;
	}

	switch(forChannel)
	{
	case 1:
		ChMask =0xFC;
		break;
	case 2:
		ChMask =0xF3;
		break;
	case 4:
		ChMask =0xCF;
		break;
	case 8:
		ChMask =0x3F;
		break;
	default:
		return ATD_INCORRECT_CHANNEL;
	}

	if (set && clear)
	{
		return ATD_ILLEGAL_PRESET;
	}
	else if (set)
	{
		preset= ((preset & ChMask) | (0xAA & ~(ChMask)));
	}
	else if (clear)
	{
		preset = ((preset &  ChMask) | (0x55 & ~(ChMask)));
	}
	else //no preset
	{
		preset == (preset & ChMask);
	}
	return 1;
}
	


